@@ -1,4 +1,5 @@
 from enum import Enum
+from typing import Mapping, Iterator
 
 
 class UserDataRRiotReferenceField(str, Enum):
@@ -217,750 +218,800 @@ class MultiMapListField(str, Enum):
     MAP_INFO = "map_info"
 
 
-class Reference:
+class RoborockBase(Mapping):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        self._data = data if isinstance(data, dict) else {}
+
+    def __getitem__(self, key):
+        return self._data.__getitem__(key)
+
+    def __setitem__(self, key, value):
+        return self._data.__setitem__(key, value)
+
+    def __len__(self) -> int:
+        return self._data.__len__()
+
+    def __iter__(self) -> Iterator:
+        return self._data.__iter__()
+
+    def keys(self):
+        return self._data.keys()
+
+    def values(self):
+        return self._data.values()
+
+    def __repr__(self):
+        return self._data.__repr__()
+
+    def __str__(self):
+        return self._data.__str__()
+
+    @property
+    def __dict__(self):
+        return self._data
+
+
+class Reference(RoborockBase):
+    def __init__(self, data: dict[str, any]) -> None:
+        super().__init__(data)
 
     @property
     def region(self):
-        return self.data.get(UserDataRRiotReferenceField.REGION)
+        return self.get(UserDataRRiotReferenceField.REGION)
 
     @property
     def api(self):
-        return self.data.get(UserDataRRiotReferenceField.API)
+        return self.get(UserDataRRiotReferenceField.API)
 
     @property
     def mqtt(self):
-        return self.data.get(UserDataRRiotReferenceField.MQTT)
+        return self.get(UserDataRRiotReferenceField.MQTT)
 
     @property
     def l_unknown(self):
-        return self.data.get(UserDataRRiotReferenceField.L_UNKNOWN)
+        return self.get(UserDataRRiotReferenceField.L_UNKNOWN)
 
 
-class RRiot:
+class RRiot(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def user(self):
-        return self.data.get(UserDataRRiotField.USER)
+        return self.get(UserDataRRiotField.USER)
 
     @property
     def password(self):
-        return self.data.get(UserDataRRiotField.PASSWORD)
+        return self.get(UserDataRRiotField.PASSWORD)
 
     @property
     def h_unknown(self):
-        return self.data.get(UserDataRRiotField.H_UNKNOWN)
+        return self.get(UserDataRRiotField.H_UNKNOWN)
 
     @property
     def domain(self):
-        return self.data.get(UserDataRRiotField.DOMAIN)
+        return self.get(UserDataRRiotField.DOMAIN)
 
     @property
     def reference(self) -> Reference:
-        return Reference(self.data.get(UserDataRRiotField.REFERENCE))
+        return Reference(self.get(UserDataRRiotField.REFERENCE))
 
 
-class UserData:
+class UserData(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def uid(self):
-        return self.data.get(UserDataField.UID)
+        return self.get(UserDataField.UID)
 
     @property
     def token_type(self):
-        return self.data.get(UserDataField.TOKEN_TYPE)
+        return self.get(UserDataField.TOKEN_TYPE)
 
     @property
     def token(self):
-        return self.data.get(UserDataField.TOKEN)
+        return self.get(UserDataField.TOKEN)
 
     @property
     def rr_uid(self):
-        return self.data.get(UserDataField.RR_UID)
+        return self.get(UserDataField.RR_UID)
 
     @property
     def region(self):
-        return self.data.get(UserDataField.REGION)
+        return self.get(UserDataField.REGION)
 
     @property
     def country_code(self):
-        return self.data.get(UserDataField.COUNTRY_CODE)
+        return self.get(UserDataField.COUNTRY_CODE)
 
     @property
     def country(self):
-        return self.data.get(UserDataField.COUNTRY)
+        return self.get(UserDataField.COUNTRY)
 
     @property
     def nickname(self):
-        return self.data.get(UserDataField.NICKNAME)
+        return self.get(UserDataField.NICKNAME)
 
     @property
     def rriot(self) -> RRiot:
-        return RRiot(self.data.get(UserDataField.RRIOT))
+        return RRiot(self.get(UserDataField.RRIOT))
 
     @property
     def tuya_device_state(self):
-        return self.data.get(UserDataField.TUYA_DEVICE_STATE)
+        return self.get(UserDataField.TUYA_DEVICE_STATE)
 
     @property
     def avatar_url(self):
-        return self.data.get(UserDataField.AVATAR_URL)
+        return self.get(UserDataField.AVATAR_URL)
 
 
-class HomeDataProductSchema:
+class LoginData(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
+
+    @property
+    def user_data(self):
+        user_data = self.get("user_data")
+        if user_data:
+            return UserData(user_data)
+
+    @property
+    def home_data(self):
+        home_data = self.get("home_data")
+        if home_data:
+            return HomeData(home_data)
+
+    @property
+    def email(self):
+        return self.get("email")
+
+
+class HomeDataProductSchema(RoborockBase):
+    def __init__(self, data: dict[str, any]) -> None:
+        super().__init__(data)
 
     @property
     def id(self):
-        return self.data.get(HomeDataProductSchemaField.ID)
+        return self.get(HomeDataProductSchemaField.ID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataProductSchemaField.NAME)
+        return self.get(HomeDataProductSchemaField.NAME)
 
     @property
     def code(self):
-        return self.data.get(HomeDataProductSchemaField.CODE)
+        return self.get(HomeDataProductSchemaField.CODE)
 
     @property
     def mode(self):
-        return self.data.get(HomeDataProductSchemaField.MODE)
+        return self.get(HomeDataProductSchemaField.MODE)
 
     @property
     def type(self):
-        return self.data.get(HomeDataProductSchemaField.TYPE)
+        return self.get(HomeDataProductSchemaField.TYPE)
 
     @property
     def product_property(self):
-        return self.data.get(HomeDataProductSchemaField.PROPERTY)
+        return self.get(HomeDataProductSchemaField.PROPERTY)
 
     @property
     def desc(self):
-        return self.data.get(HomeDataProductSchemaField.DESC)
+        return self.get(HomeDataProductSchemaField.DESC)
 
 
-class HomeDataProduct:
+class HomeDataProduct(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def id(self):
-        return self.data.get(HomeDataProductField.ID)
+        return self.get(HomeDataProductField.ID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataProductField.NAME)
+        return self.get(HomeDataProductField.NAME)
 
     @property
     def code(self):
-        return self.data.get(HomeDataProductField.CODE)
+        return self.get(HomeDataProductField.CODE)
 
     @property
     def model(self):
-        return self.data.get(HomeDataProductField.MODEL)
+        return self.get(HomeDataProductField.MODEL)
 
     @property
     def iconurl(self):
-        return self.data.get(HomeDataProductField.ICONURL)
+        return self.get(HomeDataProductField.ICONURL)
 
     @property
     def attribute(self):
-        return self.data.get(HomeDataProductField.ATTRIBUTE)
+        return self.get(HomeDataProductField.ATTRIBUTE)
 
     @property
     def capability(self):
-        return self.data.get(HomeDataProductField.CAPABILITY)
+        return self.get(HomeDataProductField.CAPABILITY)
 
     @property
     def category(self):
-        return self.data.get(HomeDataProductField.CATEGORY)
+        return self.get(HomeDataProductField.CATEGORY)
 
     @property
     def schema(self) -> list[HomeDataProductSchema]:
-        return [HomeDataProductSchema(schema) for schema in self.data.get(HomeDataProductField.SCHEMA)]
+        return [HomeDataProductSchema(schema) for schema in self.get(HomeDataProductField.SCHEMA)]
 
 
-class HomeDataDeviceStatus:
+class HomeDataDeviceStatus(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def id(self):
-        return self.data.get(HomeDataDeviceStatusField.ID)
+        return self.get(HomeDataDeviceStatusField.ID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataDeviceStatusField.NAME)
+        return self.get(HomeDataDeviceStatusField.NAME)
 
     @property
     def code(self):
-        return self.data.get(HomeDataDeviceStatusField.CODE)
+        return self.get(HomeDataDeviceStatusField.CODE)
 
     @property
     def model(self):
-        return self.data.get(HomeDataDeviceStatusField.MODEL)
+        return self.get(HomeDataDeviceStatusField.MODEL)
 
     @property
     def icon_url(self):
-        return self.data.get(HomeDataDeviceStatusField.ICON_URL)
+        return self.get(HomeDataDeviceStatusField.ICON_URL)
 
     @property
     def attribute(self):
-        return self.data.get(HomeDataDeviceStatusField.ATTRIBUTE)
+        return self.get(HomeDataDeviceStatusField.ATTRIBUTE)
 
     @property
     def capability(self):
-        return self.data.get(HomeDataDeviceStatusField.CAPABILITY)
+        return self.get(HomeDataDeviceStatusField.CAPABILITY)
 
     @property
     def category(self):
-        return self.data.get(HomeDataDeviceStatusField.CATEGORY)
+        return self.get(HomeDataDeviceStatusField.CATEGORY)
 
     @property
     def schema(self):
-        return self.data.get(HomeDataDeviceStatusField.SCHEMA)
+        return self.get(HomeDataDeviceStatusField.SCHEMA)
 
 
-class HomeDataDevice:
-
+class HomeDataDevice(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def duid(self) -> str:
-        return self.data.get(HomeDataDeviceField.DUID)
+        return self.get(HomeDataDeviceField.DUID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataDeviceField.NAME)
+        return self.get(HomeDataDeviceField.NAME)
 
     @property
     def attribute(self):
-        return self.data.get(HomeDataDeviceField.ATTRIBUTE)
+        return self.get(HomeDataDeviceField.ATTRIBUTE)
 
     @property
     def activetime(self):
-        return self.data.get(HomeDataDeviceField.ACTIVETIME)
+        return self.get(HomeDataDeviceField.ACTIVETIME)
 
     @property
     def local_key(self) -> str:
-        return self.data.get(HomeDataDeviceField.LOCAL_KEY)
+        return self.get(HomeDataDeviceField.LOCAL_KEY)
 
     @property
     def runtime_env(self):
-        return self.data.get(HomeDataDeviceField.RUNTIME_ENV)
+        return self.get(HomeDataDeviceField.RUNTIME_ENV)
 
     @property
     def time_zone_id(self):
-        return self.data.get(HomeDataDeviceField.TIME_ZONE_ID)
+        return self.get(HomeDataDeviceField.TIME_ZONE_ID)
 
     @property
     def icon_url(self):
-        return self.data.get(HomeDataDeviceField.ICON_URL)
+        return self.get(HomeDataDeviceField.ICON_URL)
 
     @property
     def product_id(self):
-        return self.data.get(HomeDataDeviceField.PRODUCT_ID)
+        return self.get(HomeDataDeviceField.PRODUCT_ID)
 
     @property
     def lon(self):
-        return self.data.get(HomeDataDeviceField.LON)
+        return self.get(HomeDataDeviceField.LON)
 
     @property
     def lat(self):
-        return self.data.get(HomeDataDeviceField.LAT)
+        return self.get(HomeDataDeviceField.LAT)
 
     @property
     def share(self):
-        return self.data.get(HomeDataDeviceField.SHARE)
+        return self.get(HomeDataDeviceField.SHARE)
 
     @property
     def share_time(self):
-        return self.data.get(HomeDataDeviceField.SHARE_TIME)
+        return self.get(HomeDataDeviceField.SHARE_TIME)
 
     @property
     def online(self):
-        return self.data.get(HomeDataDeviceField.ONLINE)
+        return self.get(HomeDataDeviceField.ONLINE)
 
     @property
     def fv(self):
-        return self.data.get(HomeDataDeviceField.FV)
+        return self.get(HomeDataDeviceField.FV)
 
     @property
     def pv(self):
-        return self.data.get(HomeDataDeviceField.PV)
+        return self.get(HomeDataDeviceField.PV)
 
     @property
     def room_id(self):
-        return self.data.get(HomeDataDeviceField.ROOM_ID)
+        return self.get(HomeDataDeviceField.ROOM_ID)
 
     @property
     def tuya_uuid(self):
-        return self.data.get(HomeDataDeviceField.TUYA_UUID)
+        return self.get(HomeDataDeviceField.TUYA_UUID)
 
     @property
     def tuya_migrated(self):
-        return self.data.get(HomeDataDeviceField.TUYA_MIGRATED)
+        return self.get(HomeDataDeviceField.TUYA_MIGRATED)
 
     @property
     def extra(self):
-        return self.data.get(HomeDataDeviceField.EXTRA)
+        return self.get(HomeDataDeviceField.EXTRA)
 
     @property
     def sn(self):
-        return self.data.get(HomeDataDeviceField.SN)
+        return self.get(HomeDataDeviceField.SN)
 
     @property
     def feature_set(self):
-        return self.data.get(HomeDataDeviceField.FEATURE_SET)
+        return self.get(HomeDataDeviceField.FEATURE_SET)
 
     @property
     def new_feature_set(self):
-        return self.data.get(HomeDataDeviceField.NEW_FEATURE_SET)
+        return self.get(HomeDataDeviceField.NEW_FEATURE_SET)
 
     @property
     def device_status(self) -> HomeDataDeviceStatus:
-        return HomeDataDeviceStatus(self.data.get(HomeDataDeviceField.DEVICE_STATUS))
+        return HomeDataDeviceStatus(self.get(HomeDataDeviceField.DEVICE_STATUS))
 
     @property
     def silent_ota_switch(self):
-        return self.data.get(HomeDataDeviceField.SILENT_OTA_SWITCH)
+        return self.get(HomeDataDeviceField.SILENT_OTA_SWITCH)
 
 
-class HomeDataRoom:
+class HomeDataRoom(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def id(self):
-        return self.data.get(HomeDataRoomField.ID)
+        return self.get(HomeDataRoomField.ID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataRoomField.NAME)
+        return self.get(HomeDataRoomField.NAME)
 
 
-class HomeData:
+class HomeData(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def id(self):
-        return self.data.get(HomeDataField.ID)
+        return self.get(HomeDataField.ID)
 
     @property
     def name(self):
-        return self.data.get(HomeDataField.NAME)
+        return self.get(HomeDataField.NAME)
 
     @property
     def lon(self):
-        return self.data.get(HomeDataField.LON)
+        return self.get(HomeDataField.LON)
 
     @property
     def lat(self):
-        return self.data.get(HomeDataField.LAT)
+        return self.get(HomeDataField.LAT)
 
     @property
     def geo_name(self):
-        return self.data.get(HomeDataField.GEO_NAME)
+        return self.get(HomeDataField.GEO_NAME)
 
     @property
     def products(self) -> list[HomeDataProduct]:
-        return [HomeDataProduct(product) for product in self.data.get(HomeDataField.PRODUCTS)]
+        return [HomeDataProduct(product) for product in self.get(HomeDataField.PRODUCTS)]
 
     @property
     def devices(self) -> list[HomeDataDevice]:
-        return [HomeDataDevice(device) for device in self.data.get(HomeDataField.DEVICES)]
+        return [HomeDataDevice(device) for device in self.get(HomeDataField.DEVICES)]
 
     @property
     def received_devices(self) -> list[HomeDataDevice]:
-        return [HomeDataDevice(device) for device in self.data.get(HomeDataField.RECEIVED_DEVICES)]
+        return [HomeDataDevice(device) for device in self.get(HomeDataField.RECEIVED_DEVICES)]
 
     @property
     def rooms(self) -> list[HomeDataRoom]:
-        return [HomeDataRoom(room) for room in self.data.get(HomeDataField.ROOMS)]
+        return [HomeDataRoom(room) for room in self.get(HomeDataField.ROOMS)]
 
 
-class Status:
+class Status(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def msg_ver(self):
-        return self.data.get(StatusField.MSG_VER)
+        return self.get(StatusField.MSG_VER)
 
     @property
     def msg_seq(self):
-        return self.data.get(StatusField.MSG_SEQ)
+        return self.get(StatusField.MSG_SEQ)
 
     @property
     def state(self):
-        return self.data.get(StatusField.STATE)
+        return self.get(StatusField.STATE)
 
     @property
     def battery(self):
-        return self.data.get(StatusField.BATTERY)
+        return self.get(StatusField.BATTERY)
 
     @property
     def clean_time(self):
-        return self.data.get(StatusField.CLEAN_TIME)
+        return self.get(StatusField.CLEAN_TIME)
 
     @property
     def clean_area(self):
-        return self.data.get(StatusField.CLEAN_AREA)
+        return self.get(StatusField.CLEAN_AREA)
 
     @property
     def error_code(self) -> int:
-        return self.data.get(StatusField.ERROR_CODE)
+        return self.get(StatusField.ERROR_CODE)
 
     @property
     def map_present(self):
-        return self.data.get(StatusField.MAP_PRESENT)
+        return self.get(StatusField.MAP_PRESENT)
 
     @property
     def in_cleaning(self):
-        return self.data.get(StatusField.IN_CLEANING)
+        return self.get(StatusField.IN_CLEANING)
 
     @property
     def in_returning(self):
-        return self.data.get(StatusField.IN_RETURNING)
+        return self.get(StatusField.IN_RETURNING)
 
     @property
     def in_fresh_state(self):
-        return self.data.get(StatusField.IN_FRESH_STATE)
+        return self.get(StatusField.IN_FRESH_STATE)
 
     @property
     def lab_status(self):
-        return self.data.get(StatusField.LAB_STATUS)
+        return self.get(StatusField.LAB_STATUS)
 
     @property
     def water_box_status(self):
-        return self.data.get(StatusField.WATER_BOX_STATUS)
+        return self.get(StatusField.WATER_BOX_STATUS)
 
     @property
     def back_type(self):
-        return self.data.get(StatusField.BACK_TYPE)
+        return self.get(StatusField.BACK_TYPE)
 
     @property
     def wash_phase(self):
-        return self.data.get(StatusField.WASH_PHASE)
+        return self.get(StatusField.WASH_PHASE)
 
     @property
     def wash_ready(self):
-        return self.data.get(StatusField.WASH_READY)
+        return self.get(StatusField.WASH_READY)
 
     @property
     def fan_power(self):
-        return self.data.get(StatusField.FAN_POWER)
+        return self.get(StatusField.FAN_POWER)
 
     @property
     def dnd_enabled(self):
-        return self.data.get(StatusField.DND_ENABLED)
+        return self.get(StatusField.DND_ENABLED)
 
     @property
     def map_status(self):
-        return self.data.get(StatusField.MAP_STATUS)
+        return self.get(StatusField.MAP_STATUS)
 
     @property
     def is_locating(self):
-        return self.data.get(StatusField.IS_LOCATING)
+        return self.get(StatusField.IS_LOCATING)
 
     @property
     def lock_status(self):
-        return self.data.get(StatusField.LOCK_STATUS)
+        return self.get(StatusField.LOCK_STATUS)
 
     @property
     def water_box_mode(self):
-        return self.data.get(StatusField.WATER_BOX_MODE)
+        return self.get(StatusField.WATER_BOX_MODE)
 
     @property
     def water_box_carriage_status(self):
-        return self.data.get(StatusField.WATER_BOX_CARRIAGE_STATUS)
+        return self.get(StatusField.WATER_BOX_CARRIAGE_STATUS)
 
     @property
     def mop_forbidden_enable(self):
-        return self.data.get(StatusField.MOP_FORBIDDEN_ENABLE)
+        return self.get(StatusField.MOP_FORBIDDEN_ENABLE)
 
     @property
     def camera_status(self):
-        return self.data.get(StatusField.CAMERA_STATUS)
+        return self.get(StatusField.CAMERA_STATUS)
 
     @property
     def is_exploring(self):
-        return self.data.get(StatusField.IS_EXPLORING)
+        return self.get(StatusField.IS_EXPLORING)
 
     @property
     def home_sec_status(self):
-        return self.data.get(StatusField.HOME_SEC_STATUS)
+        return self.get(StatusField.HOME_SEC_STATUS)
 
     @property
     def home_sec_enable_password(self):
-        return self.data.get(StatusField.HOME_SEC_ENABLE_PASSWORD)
+        return self.get(StatusField.HOME_SEC_ENABLE_PASSWORD)
 
     @property
     def adbumper_status(self):
-        return self.data.get(StatusField.ADBUMPER_STATUS)
+        return self.get(StatusField.ADBUMPER_STATUS)
 
     @property
     def water_shortage_status(self):
-        return self.data.get(StatusField.WATER_SHORTAGE_STATUS)
+        return self.get(StatusField.WATER_SHORTAGE_STATUS)
 
     @property
     def dock_type(self):
-        return self.data.get(StatusField.DOCK_TYPE)
+        return self.get(StatusField.DOCK_TYPE)
 
     @property
     def dust_collection_status(self):
-        return self.data.get(StatusField.DUST_COLLECTION_STATUS)
+        return self.get(StatusField.DUST_COLLECTION_STATUS)
 
     @property
     def auto_dust_collection(self):
-        return self.data.get(StatusField.AUTO_DUST_COLLECTION)
+        return self.get(StatusField.AUTO_DUST_COLLECTION)
 
     @property
     def avoid_count(self):
-        return self.data.get(StatusField.AVOID_COUNT)
+        return self.get(StatusField.AVOID_COUNT)
 
     @property
     def mop_mode(self):
-        return self.data.get(StatusField.MOP_MODE)
+        return self.get(StatusField.MOP_MODE)
 
     @property
     def debug_mode(self):
-        return self.data.get(StatusField.DEBUG_MODE)
+        return self.get(StatusField.DEBUG_MODE)
 
     @property
     def collision_avoid_status(self):
-        return self.data.get(StatusField.COLLISION_AVOID_STATUS)
+        return self.get(StatusField.COLLISION_AVOID_STATUS)
 
     @property
     def switch_map_mode(self):
-        return self.data.get(StatusField.SWITCH_MAP_MODE)
+        return self.get(StatusField.SWITCH_MAP_MODE)
 
     @property
     def dock_error_status(self):
-        return self.data.get(StatusField.DOCK_ERROR_STATUS)
+        return self.get(StatusField.DOCK_ERROR_STATUS)
 
     @property
     def charge_status(self):
-        return self.data.get(StatusField.CHARGE_STATUS)
+        return self.get(StatusField.CHARGE_STATUS)
 
     @property
     def unsave_map_reason(self):
-        return self.data.get(StatusField.UNSAVE_MAP_REASON)
+        return self.get(StatusField.UNSAVE_MAP_REASON)
 
     @property
     def unsave_map_flag(self):
-        return self.data.get(StatusField.UNSAVE_MAP_FLAG)
+        return self.get(StatusField.UNSAVE_MAP_FLAG)
 
 
-class DNDTimer:
+class DNDTimer(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def start_hour(self):
-        return self.data.get(DNDTimerField.START_HOUR)
+        return self.get(DNDTimerField.START_HOUR)
 
     @property
     def start_minute(self):
-        return self.data.get(DNDTimerField.START_MINUTE)
+        return self.get(DNDTimerField.START_MINUTE)
 
     @property
     def end_hour(self):
-        return self.data.get(DNDTimerField.END_HOUR)
+        return self.get(DNDTimerField.END_HOUR)
 
     @property
     def end_minute(self):
-        return self.data.get(DNDTimerField.END_MINUTE)
+        return self.get(DNDTimerField.END_MINUTE)
 
     @property
     def enabled(self):
-        return self.data.get(DNDTimerField.ENABLED)
+        return self.get(DNDTimerField.ENABLED)
 
 
-class CleanSummary:
+class CleanSummary(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def clean_time(self):
-        return self.data.get(CleanSummaryField.CLEAN_TIME)
+        return self.get(CleanSummaryField.CLEAN_TIME)
 
     @property
     def clean_area(self):
-        return self.data.get(CleanSummaryField.CLEAN_AREA)
+        return self.get(CleanSummaryField.CLEAN_AREA)
 
     @property
     def clean_count(self):
-        return self.data.get(CleanSummaryField.CLEAN_COUNT)
+        return self.get(CleanSummaryField.CLEAN_COUNT)
 
     @property
     def dust_collection_count(self):
-        return self.data.get(CleanSummaryField.DUST_COLLECTION_COUNT)
+        return self.get(CleanSummaryField.DUST_COLLECTION_COUNT)
 
     @property
     def records(self) -> list[int]:
-        return self.data.get(CleanSummaryField.RECORDS)
+        return self.get(CleanSummaryField.RECORDS)
 
 
-class CleanRecord:
+class CleanRecord(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def begin(self):
-        return self.data.get(CleanRecordField.BEGIN)
+        return self.get(CleanRecordField.BEGIN)
 
     @property
     def end(self):
-        return self.data.get(CleanRecordField.END)
+        return self.get(CleanRecordField.END)
 
     @property
     def duration(self):
-        return self.data.get(CleanRecordField.DURATION)
+        return self.get(CleanRecordField.DURATION)
 
     @property
     def area(self):
-        return self.data.get(CleanRecordField.AREA)
+        return self.get(CleanRecordField.AREA)
 
     @property
     def error(self):
-        return self.data.get(CleanRecordField.ERROR)
+        return self.get(CleanRecordField.ERROR)
 
     @property
     def complete(self):
-        return self.data.get(CleanRecordField.COMPLETE)
+        return self.get(CleanRecordField.COMPLETE)
 
     @property
     def start_type(self):
-        return self.data.get(CleanRecordField.START_TYPE)
+        return self.get(CleanRecordField.START_TYPE)
 
     @property
     def clean_type(self):
-        return self.data.get(CleanRecordField.CLEAN_TYPE)
+        return self.get(CleanRecordField.CLEAN_TYPE)
 
     @property
     def finish_reason(self):
-        return self.data.get(CleanRecordField.FINISH_REASON)
+        return self.get(CleanRecordField.FINISH_REASON)
 
     @property
     def dust_collection_status(self):
-        return self.data.get(CleanRecordField.DUST_COLLECTION_STATUS)
+        return self.get(CleanRecordField.DUST_COLLECTION_STATUS)
 
     @property
     def avoid_count(self):
-        return self.data.get(CleanRecordField.AVOID_COUNT)
+        return self.get(CleanRecordField.AVOID_COUNT)
 
     @property
     def wash_count(self):
-        return self.data.get(CleanRecordField.WASH_COUNT)
+        return self.get(CleanRecordField.WASH_COUNT)
 
     @property
     def map_flag(self):
-        return self.data.get(CleanRecordField.MAP_FLAG)
+        return self.get(CleanRecordField.MAP_FLAG)
 
 
-class Consumable:
+class Consumable(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def main_brush_work_time(self):
-        return self.data.get(ConsumableField.MAIN_BRUSH_WORK_TIME)
+        return self.get(ConsumableField.MAIN_BRUSH_WORK_TIME)
 
     @property
     def side_brush_work_time(self):
-        return self.data.get(ConsumableField.SIDE_BRUSH_WORK_TIME)
+        return self.get(ConsumableField.SIDE_BRUSH_WORK_TIME)
 
     @property
     def filter_work_time(self):
-        return self.data.get(ConsumableField.FILTER_WORK_TIME)
+        return self.get(ConsumableField.FILTER_WORK_TIME)
 
     @property
     def filter_element_work_time(self):
-        return self.data.get(ConsumableField.FILTER_ELEMENT_WORK_TIME)
+        return self.get(ConsumableField.FILTER_ELEMENT_WORK_TIME)
 
     @property
     def sensor_dirty_time(self):
-        return self.data.get(ConsumableField.SENSOR_DIRTY_TIME)
+        return self.get(ConsumableField.SENSOR_DIRTY_TIME)
 
     @property
     def strainer_work_times(self):
-        return self.data.get(ConsumableField.STRAINER_WORK_TIMES)
+        return self.get(ConsumableField.STRAINER_WORK_TIMES)
 
     @property
     def dust_collection_work_times(self):
-        return self.data.get(ConsumableField.DUST_COLLECTION_WORK_TIMES)
+        return self.get(ConsumableField.DUST_COLLECTION_WORK_TIMES)
 
     @property
     def cleaning_brush_work_times(self):
-        return self.data.get(ConsumableField.CLEANING_BRUSH_WORK_TIMES)
+        return self.get(ConsumableField.CLEANING_BRUSH_WORK_TIMES)
 
 
-class MultiMapsListMapInfoBakMaps:
-
+class MultiMapsListMapInfoBakMaps(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def mapflag(self):
-        return self.data.get(MultiMapListMapInfoBakMapsField.MAPFLAG)
+        return self.get(MultiMapListMapInfoBakMapsField.MAPFLAG)
 
     @property
     def add_time(self):
-        return self.data.get(MultiMapListMapInfoBakMapsField.ADD_TIME)
+        return self.get(MultiMapListMapInfoBakMapsField.ADD_TIME)
 
 
-class MultiMapsListMapInfo:
-
+class MultiMapsListMapInfo(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def mapflag(self):
-        return self.data.get(MultiMapListMapInfoField.MAPFLAG)
+        return self.get(MultiMapListMapInfoField.MAPFLAG)
 
     @property
     def add_time(self):
-        return self.data.get(MultiMapListMapInfoField.ADD_TIME)
+        return self.get(MultiMapListMapInfoField.ADD_TIME)
 
     @property
     def length(self):
-        return self.data.get(MultiMapListMapInfoField.LENGTH)
+        return self.get(MultiMapListMapInfoField.LENGTH)
 
     @property
     def name(self):
-        return self.data.get(MultiMapListMapInfoField.NAME)
+        return self.get(MultiMapListMapInfoField.NAME)
 
     @property
     def bak_maps(self):
-        return [MultiMapsListMapInfoBakMaps(bak_maps) for bak_maps in self.data.get(MultiMapListMapInfoField.BAK_MAPS)]
+        return [MultiMapsListMapInfoBakMaps(bak_maps) for bak_maps in self.get(MultiMapListMapInfoField.BAK_MAPS)]
 
 
-class MultiMapsList:
-
+class MultiMapsList(RoborockBase):
     def __init__(self, data: dict[str, any]) -> None:
-        self.data = data if isinstance(data, dict) else {}
+        super().__init__(data)
 
     @property
     def max_multi_map(self):
-        return self.data.get(MultiMapListField.MAX_MULTI_MAP)
+        return self.get(MultiMapListField.MAX_MULTI_MAP)
 
     @property
     def max_bak_map(self):
-        return self.data.get(MultiMapListField.MAX_BAK_MAP)
+        return self.get(MultiMapListField.MAX_BAK_MAP)
 
     @property
     def multi_map_count(self):
-        return self.data.get(MultiMapListField.MULTI_MAP_COUNT)
+        return self.get(MultiMapListField.MULTI_MAP_COUNT)
 
     @property
     def map_info(self):
-        return [MultiMapsListMapInfo(map_info) for map_info in self.data.get(MultiMapListField.MAP_INFO)]
+        return [MultiMapsListMapInfo(map_info) for map_info in self.get(MultiMapListField.MAP_INFO)]
